#!/usr/bin/env gnuplot

set terminal pdf mono font 'Bitstream Charter'
set termoption dash

set output 'overhead.pdf'

set key at 0.8*10**4,0.735 box linetype -1 linewidth 1.000 width 1.5
set bars 2.0

set boxwidth 0.15

set xrange [10:10**4]
set logscale x

set xtic auto
set ytic auto

set xlabel 'Vector Size'
set ylabel 'Overhead Ratio'

# overhead of a node
o = 16.0
# size of a trie pointer
p = 8.0
T = 8.0

max(a, b) = (a > b) ? a : b

h(x, M) = (x == 0) ? 0 : max(ceil(log(x)/log(M)) - 1, 0)
rec(x, i, M) = (i == 0) ? o : 1.0*floor(x/(M**i)) * (p + o) + rec(x, i-1, M)
overhead(x, M) = rec(x, h(x, M), M)
total(x, M) = overhead(x, M)/(overhead(x, M) + x*T)
full(x, M) = (p*(x - M)+ o*(x - 1))/(M - 1)
total2(x, M) = full(x, M)/(full(x, M) + x*T)

plot total(x, 2) t '$M=2$',\
     total(x, 4) t '$M=4$',\
     total(x, 8) t '$M=8$',\
     total(x, 16) t '$M=16$',\
     total(x, 32) t '$M=32$',\
     total2(x, 32) t 'foo',\
     total2(x, 2) t 'bar',\
     total2(x, 16) t 'quux'

## set terminal epslatex
## set output 'overhead.tex'
## replot
